import json
import logging

from aiohttp import web
from aiohttp import WSMsgType
from pydantic.error_wrappers import ValidationError

from pongy.models import WsCommand
from pongy.models import WsCookie
from pongy.models import WsErrorEvent
from pongy.models import WsErrorEventPayload
from pongy.models import WsEvent
from pongy.server.game import GamePool
from pongy.server.game import Player

logger = logging.getLogger(__name__)


class WebsocketHandler(web.View):
    async def get(self) -> web.WebSocketResponse:
        ws = web.WebSocketResponse()
        await ws.prepare(self.request)
        try:
            cookie = WsCookie(**self.request.cookies)
            player = Player(uuid=cookie.player_id, ws=ws)
            async with GamePool(player):
                async for message in ws:
                    if message.type == WSMsgType.TEXT:
                        command = WsCommand(**json.loads(message.data))
                        player.move_racket(command.payload.direction)
        except Exception as err:
            await self.send_error(err, ws)
            return ws
        else:
            logger.debug("Websocket connection closed")
            return ws

    @staticmethod
    async def send_error(error: Exception, ws: web.WebSocketResponse) -> None:
        if isinstance(error, ValidationError):
            message = str(
                ";".join(" ".join(map(str, e.values())) for e in error.errors())
            )
        else:
            message = str(error)
        logger.warning(message)
        await ws.send_json(
            WsEvent(
                data=WsErrorEvent(payload=WsErrorEventPayload(message=message))
            ).dict()
        )


class IndexHandler(web.View):
    async def get(self) -> web.Response:
        return web.json_response({})


def get_application() -> web.Application:
    app = web.Application()
    app.router.add_route("GET", "/", IndexHandler)
    app.router.add_route("GET", "/ws", WebsocketHandler)
    return app



import asyncio
import logging
import queue
import threading
import time
import uuid
from contextlib import suppress

import aiohttp
import pygame

from pongy import settings
from pongy.models import MoveDirection
from pongy.models import WsCommand
from pongy.models import WsCommandMovePayload
from pongy.models import WsEvent
from pongy.ui.ball import Ball
from pongy.ui.racket import BottomRacketWidget
from pongy.ui.racket import LeftRacketWidget
from pongy.ui.racket import RightRacketWidget
from pongy.ui.racket import TopRacketWidget
from pongy.ui.score import BottomScoreWidget
from pongy.ui.score import LeftScoreWidget
from pongy.ui.score import RightScoreWidget
from pongy.ui.score import TopScoreWidget

logger = logging.getLogger(__name__)


class Exit:
    pass


def run_app(host: str, port: int):
    pygame.init()
    quit = False
    input_queue: queue.Queue = queue.Queue()
    output_queue: queue.Queue = queue.Queue()
    player_id = str(uuid.uuid4())

    def ws_connection():
        async def _process_output_queue(ws):
            while True:
                await asyncio.sleep(0.01)
                with suppress(queue.Empty):
                    await ws.send_json(output_queue.get_nowait().dict())

        async def _handle():
            url = f"ws://{host}:{port}/ws"
            try:
                async with aiohttp.ClientSession() as session:
                    async with session.ws_connect(
                        url,
                        heartbeat=settings.WS_HEARTBEAT_TIMEOUT,
                        headers={"Cookie": f"player_id={player_id}"},
                    ) as ws:
                        task = asyncio.ensure_future(_process_output_queue(ws))
                        async for msg in ws:
                            if quit or msg.type == aiohttp.WSMsgType.ERROR:
                                task.cancel()
                                break
                            if msg.type == aiohttp.WSMsgType.TEXT:
                                ws_event = WsEvent.parse_raw(msg.data)
                                input_queue.put(ws_event)
            except aiohttp.ClientConnectionError:
                logger.error("Connection error")
                input_queue.put(Exit())

        loop = asyncio.new_event_loop()
        loop.run_until_complete(_handle())

    keep_connection = threading.Thread(target=ws_connection)
    keep_connection.start()

    surface = pygame.display.set_mode((settings.BOARD_SIZE, settings.BOARD_SIZE))
    surface.fill(settings.BOARD_COLOR)

    while not quit:
        time.sleep(0.01)
        while not input_queue.empty():
            ws_event = input_queue.get_nowait()
            if isinstance(ws_event, Exit):
                quit = True
                break
            surface.fill(settings.BOARD_COLOR)
            for player, racket_widget, score_widget in zip(
                ws_event.data.payload.players,
                (
                    BottomRacketWidget,
                    TopRacketWidget,
                    LeftRacketWidget,
                    RightRacketWidget,
                ),
                (BottomScoreWidget, TopScoreWidget, LeftScoreWidget, RightScoreWidget),
            ):
                racket_widget(player.racket_position).draw(surface)
                score_widget(player.score).draw(surface)

            ball_position = ws_event.data.payload.ball_position
            Ball(ball_position).draw(surface)
            pygame.display.flip()
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                quit = True
            elif event.type == pygame.KEYDOWN:
                if event.key == pygame.K_q:
                    quit = True
        keys = pygame.key.get_pressed()
        if keys[pygame.K_LEFT] or keys[pygame.K_UP]:
            ws_event = WsCommand(
                payload=WsCommandMovePayload(direction=MoveDirection.LEFT)
            )
            output_queue.put(ws_event)
        if keys[pygame.K_RIGHT] or keys[pygame.K_DOWN]:
            ws_event = WsCommand(
                payload=WsCommandMovePayload(direction=MoveDirection.RIGHT)
            )
            output_queue.put(ws_event)
    pygame.quit()



import pygame

from pongy import settings


class BaseRacketWidget:
    def __init__(self, position):
        self._position = position

    def draw(self, surface):
        pass


class HorizontalRacketWidget(BaseRacketWidget):
    def __init__(self, position):
        super().__init__(position)
        self._width = settings.RACKET_LENGTH
        self._height = settings.RACKET_HEIGHT


class VerticalRacketWidget(BaseRacketWidget):
    def __init__(self, position):
        super().__init__(position)
        self._width = settings.RACKET_HEIGHT
        self._height = settings.RACKET_LENGTH


class TopRacketWidget(HorizontalRacketWidget):
    def draw(self, surface):
        pygame.draw.rect(
            surface,
            settings.RACKET_COLOR,
            pygame.Rect(
                self._position,
                0,
                self._width,
                self._height,
            ),
        )


class LeftRacketWidget(VerticalRacketWidget):
    def draw(self, surface):
        pygame.draw.rect(
            surface,
            settings.RACKET_COLOR,
            pygame.Rect(
                0,
                self._position,
                self._width,
                self._height,
            ),
        )


class RightRacketWidget(VerticalRacketWidget):
    def draw(self, surface):
        pygame.draw.rect(
            surface,
            settings.RACKET_COLOR,
            pygame.Rect(
                settings.BOARD_SIZE - settings.RACKET_HEIGHT,
                self._position,
                self._width,
                self._height,
            ),
        )


class BottomRacketWidget(HorizontalRacketWidget):
    def draw(self, surface):
        pygame.draw.rect(
            surface,
            settings.RACKET_COLOR,
            pygame.Rect(
                self._position,
                settings.BOARD_SIZE - settings.RACKET_HEIGHT,
                self._width,
                self._height,
            ),
        )



from enum import IntEnum
from typing import Literal

from pydantic import BaseModel


class MoveDirection(IntEnum):
    LEFT = 1
    RIGHT = 2


class WsErrorEventPayload(BaseModel):
    message: str


class WsErrorEvent(BaseModel):
    event: Literal["error"] = "error"
    payload: WsErrorEventPayload


class WsPlayer(BaseModel):
    uuid: str
    score: int
    racket_position: int


class WsGameStatePayload(BaseModel):
    players: list[WsPlayer]
    ball_position: tuple[int, int]


class WsGameStateEvent(BaseModel):
    event: Literal["game_state"] = "game_state"
    payload: WsGameStatePayload


class WsEvent(BaseModel):
    data: WsGameStateEvent | WsErrorEvent


class WsCommandMovePayload(BaseModel):
    direction: MoveDirection

    class Config:
        use_enum_values = True


class WsCommand(BaseModel):
    command: Literal["move"] = "move"
    payload: WsCommandMovePayload


class WsCookie(BaseModel):
    player_id: str



import logging

import click
from pythonjsonlogger import jsonlogger

from pongy import settings


@click.command(context_settings=dict(help_option_names=["-h", "--help"]))
@click.option("-d", "--daemon", is_flag=True, help="Run server.")
@click.option(
    "-h",
    "--host",
    help="Hostname",
    type=click.STRING,
    default=settings.SERVER_HOST,
)
@click.option(
    "-p",
    "--port",
    help="Port",
    type=click.INT,
    default=settings.SERVER_PORT,
)
def main(daemon: bool, host: str, port: int) -> None:
    stream_handler = logging.StreamHandler()
    stream_handler.setFormatter(jsonlogger.JsonFormatter(timestamp=True))
    logging.basicConfig(level=settings.LOGGING_LEVEL, handlers=[stream_handler])
    if daemon:
        from aiohttp import web

        from pongy.server.app import get_application

        web.run_app(get_application(), host=host, port=port)
    else:
        from pongy.ui.app import run_app

        run_app(host, port)


if __name__ == "__main__":
    main()



import asyncio
import logging
import math
from random import randint
from types import TracebackType

from aiohttp import web

from pongy import settings
from pongy.models import MoveDirection
from pongy.models import WsEvent
from pongy.models import WsGameStateEvent
from pongy.models import WsGameStatePayload
from pongy.models import WsPlayer


logger = logging.getLogger(__name__)


class Player:
    def __init__(self, uuid: str, ws: web.WebSocketResponse):
        self.uuid: str = uuid
        self.ws: web.WebSocketResponse = ws
        self.score: int = 0
        self.racket_position: int = (settings.BOARD_SIZE - settings.RACKET_LENGTH) // 2

    def move_racket(self, direction: MoveDirection) -> None:
        if (
            direction == MoveDirection.LEFT
            and self.racket_position - settings.RACKET_SPEED >= 0
        ):
            self.racket_position -= settings.RACKET_SPEED
        if (
            direction == MoveDirection.RIGHT
            and self.racket_position + settings.RACKET_SPEED
            <= settings.BOARD_SIZE - settings.RACKET_LENGTH
        ):
            self.racket_position += settings.RACKET_SPEED


class Game:
    def __init__(self) -> None:
        self.players: list[Player] = []
        self.ball_position: tuple[int, int] = (
            (settings.BOARD_SIZE - settings.BALL_SIZE) // 2,
        ) * 2
        self.ball_angle: int = randint(20, 160)
        self._run_task = asyncio.ensure_future(self.run())

    def add_player(self, player: Player) -> None:
        self.players.append(player)

    def remove_player(self, player: Player) -> None:
        self.players[:] = [p for p in self.players if p.uuid != player.uuid]
        if self.is_empty:
            self._run_task.cancel()

    def move_ball(self) -> None:
        new_x = self.ball_position[0] + settings.BALL_SPEED * math.cos(
            math.radians(self.ball_angle)
        )
        new_y = self.ball_position[1] + settings.BALL_SPEED * math.sin(
            math.radians(self.ball_angle)
        )
        self.ball_position = int(new_x), int(new_y)

    def hit(self) -> None:
        new_x, new_y = self.ball_position
        if (
            len(self.players) > 0
            and new_y
            > settings.BOARD_SIZE - settings.BALL_SIZE - settings.RACKET_HEIGHT
            and self.players[0].racket_position - settings.BALL_SIZE
            < new_x
            < self.players[0].racket_position
            + settings.RACKET_LENGTH
            + settings.BALL_SIZE
        ):
            new_y = settings.BOARD_SIZE - settings.BALL_SIZE - settings.RACKET_HEIGHT
            self.ball_angle = randint(200, 340)
        if (
            len(self.players) > 1
            and new_y < settings.RACKET_HEIGHT
            and self.players[1].racket_position - settings.BALL_SIZE
            < new_x
            < self.players[1].racket_position
            + settings.RACKET_LENGTH
            + settings.BALL_SIZE
        ):
            new_y = settings.RACKET_HEIGHT
            self.ball_angle = randint(20, 160)
        if (
            len(self.players) > 2
            and new_x < settings.RACKET_HEIGHT
            and self.players[2].racket_position - settings.BALL_SIZE
            < new_y
            < self.players[2].racket_position
            + settings.RACKET_LENGTH
            + settings.BALL_SIZE
        ):
            new_x = settings.RACKET_HEIGHT
            self.ball_angle = randint(-70, 70)
        if (
            len(self.players) > 3
            and new_x
            > settings.BOARD_SIZE - settings.BALL_SIZE - settings.RACKET_HEIGHT
            and self.players[3].racket_position - settings.BALL_SIZE
            < new_y
            < self.players[3].racket_position
            + settings.RACKET_LENGTH
            + settings.BALL_SIZE
        ):
            new_x = settings.BOARD_SIZE - settings.BALL_SIZE - settings.RACKET_HEIGHT
            self.ball_angle = randint(110, 250)
        self.ball_position = int(new_x), int(new_y)

    def bounce(self) -> None:
        new_x, new_y = self.ball_position
        if new_x < 0:
            new_x = 0
            self.ball_angle = 180 - self.ball_angle
            if len(self.players) > 2:
                self.players[2].score += 1
        elif new_x > settings.BOARD_SIZE - settings.BALL_SIZE:
            new_x = settings.BOARD_SIZE - settings.BALL_SIZE
            self.ball_angle = 180 - self.ball_angle
            if len(self.players) > 3:
                self.players[3].score += 1
        if new_y < 0:
            new_y = 0
            self.ball_angle = -self.ball_angle
            if len(self.players) > 1:
                self.players[1].score += 1
        elif new_y > settings.BOARD_SIZE - settings.BALL_SIZE:
            new_y = settings.BOARD_SIZE - settings.BALL_SIZE
            self.ball_angle = -self.ball_angle
            if len(self.players) > 0:
                self.players[0].score += 1
        self.ball_position = int(new_x), int(new_y)

    def to_payload(self) -> WsGameStatePayload:
        return WsGameStatePayload(
            ball_position=self.ball_position,
            players=[
                WsPlayer(
                    uuid=player.uuid,
                    score=player.score,
                    racket_position=player.racket_position,
                )
                for player in self.players
            ],
        )

    async def run(self) -> None:
        while True:
            await self.broadcast()
            await asyncio.sleep(1 / settings.FPS)
            self.move_ball()
            self.hit()
            self.bounce()

    async def broadcast(self) -> None:
        payload = WsEvent(data=WsGameStateEvent(payload=self.to_payload()))
        await asyncio.gather(
            *(subscriber.ws.send_json(payload.dict()) for subscriber in self.players)
        )

    @property
    def is_full(self) -> bool:
        return len(self.players) == 4

    @property
    def is_empty(self) -> bool:
        return not self.players


class GamePool:
    _awaiting: Game | None = None

    def __init__(self, player: Player) -> None:
        self._player: Player = player
        self._game: Game | None = None

    async def __aenter__(self) -> Game:
        if not GamePool._awaiting:
            self._game = GamePool._awaiting = Game()
        else:
            self._game = GamePool._awaiting
        self._game.add_player(self._player)
        if self._game.is_full:
            GamePool._awaiting = None
        return self._game

    async def __aexit__(
        self,
        exc_type: type[BaseException] | None,
        exc_val: BaseException | None,
        exc_tb: TracebackType | None,
    ) -> None:
        if self._game:
            self._game.remove_player(self._player)
            if GamePool._awaiting is self._game and not self._game.players:
                GamePool._awaiting = None



import logging


RACKET_COLOR = (200, 0, 0)

BALL_COLOR = (173, 255, 47)

BOARD_COLOR = (233, 150, 122)

SCORE_COLOR = (255, 255, 255)

BOARD_SIZE = 500

RACKET_LENGTH = 40

RACKET_HEIGHT = 10

BALL_SPEED = 7

BALL_SIZE = 10

SCORE_TEXT_SIZE = 16

SCORE_TEXT_SHIFT = 100

RACKET_SPEED = 5

WS_HEARTBEAT_TIMEOUT = 10

FPS = 60

LOGGING_LEVEL = logging.DEBUG

SERVER_HOST = "0.0.0.0"

SERVER_PORT = 8888



import pygame

from pongy import settings


class Ball:
    def __init__(self, position):
        self._position = position
        self._width = settings.BALL_SIZE
        self._height = settings.BALL_SIZE

    def draw(self, surface):
        pygame.draw.rect(
            surface,
            settings.BALL_COLOR,
            pygame.Rect(
                self._position[0],
                self._position[1],
                self._width,
                self._height,
            ),
        )



import pygame

from pongy import settings


class BaseScoreWidget:
    position = (0, 0)

    def __init__(self, score):
        self._score = str(score)

    def draw(self, surface):
        font = pygame.font.SysFont("arial", settings.SCORE_TEXT_SIZE)
        text = font.render(self._score, True, settings.SCORE_COLOR)
        surface.blit(text, self.position)


class BottomScoreWidget(BaseScoreWidget):
    position = (
        settings.BOARD_SIZE // 2,
        settings.BOARD_SIZE // 2 + settings.SCORE_TEXT_SHIFT,
    )


class TopScoreWidget(BaseScoreWidget):
    position = (
        settings.BOARD_SIZE // 2,
        settings.BOARD_SIZE // 2 - settings.SCORE_TEXT_SHIFT,
    )


class LeftScoreWidget(BaseScoreWidget):
    position = (
        settings.BOARD_SIZE // 2 - settings.SCORE_TEXT_SHIFT,
        settings.BOARD_SIZE // 2,
    )


class RightScoreWidget(BaseScoreWidget):
    position = (
        settings.BOARD_SIZE // 2 + settings.SCORE_TEXT_SHIFT,
        settings.BOARD_SIZE // 2,
    )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pongy', 'pongy.server', 'pongy.ui', 'server', 'ui']

package_data = \
{'': ['*']}

modules = \
['run']
install_requires = \
['aiohttp[speedups]==3.8.1',
 'click==8.1.3',
 'pydantic==1.10.2',
 'pygame==2.1.2',
 'python-json-logger==2.0.4']

entry_points = \
{'console_scripts': ['pongy = run:main']}

setup_kwargs = {
    'name': 'pongy',
    'version': '0.1.5',
    'description': 'Ping-pong multiplayer client-server game up to 4 players over network in early development stage.',
    'long_description': '# Pongy\n\nPing-pong multiplayer client-server game up to 4 players over network in early development stage.\n\n## Requires\n\nPython 3.10\n\n## Install\n\n```\n$ pip install pongy\n```\n\nor\n\n```\n$ poetry shell\n$ poetry add pongy\n```\n\n## Run Server\n\n```\n$ pongy -d -h 0.0.0.0 -p 8888\n```\n\n## Run Client\n\n```\n$ pongy -h 192.168.1.1 -p 8888\n```\n\n![UI screenshot](https://github.com/vyalovvldmr/pongy/blob/main/screen.png?raw=true)\n',
    'author': 'Vladimir Vyalov',
    'author_email': 'vyalov.v@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/vyalovvldmr/pongy',
    'packages': packages,
    'package_data': package_data,
    'py_modules': modules,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)
